/*
	PROforma, PROgs FOnt & Raster MAnager

	Generalized drawing routines to allow bitmap drivers to implement
	a minimal set only (only Pixel and DrawMask are really essential).

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 28 September 1995
*/

#include "gstate_h"

Error _LineHorizontal(Bitmap bitmap, short aty, short fromx, short tox)
{
	BitDrawRouts *rout=bitmap->bitdraw;
	while (fromx<tox)
		rout->Pixel(bitmap,fromx++,aty);
	return ERR_OK;
}

Error _LineVertical(Bitmap bitmap, short atx, short fromy, short toy)
{
	BitDrawRouts *rout=bitmap->bitdraw;
	while (fromy<toy)
		rout->Pixel(bitmap,atx,fromy++);
	return ERR_OK;
}

Error _Block(Bitmap bitmap, short xsiz, short ysiz, short xorg, short yorg)
{
	BitDrawRouts *rout=bitmap->bitdraw;
	short xend=xorg+xsiz;

	while ((ysiz--)>0)
		rout->LineHorizontal(bitmap,yorg++,xorg,xend);
	return ERR_OK;
}

Error _TrapezoidHorizontal(Bitmap bitmap, short starty, short endy,
				pt leftstartx, pt leftincr, pt rightstartx, pt rightincr)
{
	BitDrawRouts *rout=bitmap->bitdraw;
	while (starty<endy)
	{
		rout->LineHorizontal(bitmap, starty, pt2rshort(leftstartx),
											 pt2rshort(rightstartx));
		leftstartx+=leftincr;
		rightstartx+=rightincr;
		starty++;
	}
	return ERR_OK;
}

Error _TrapezoidVertical(Bitmap bitmap, short startx, short endx,
				pt topstarty, pt topincr, pt bottomstarty, pt bottomincr)
{
	BitDrawRouts *rout=bitmap->bitdraw;
	/* adjust to make sure we draw at the pixel centres ! */
	while (startx<endx)
	{
		rout->LineVertical(bitmap, startx, pt2rshort(topstarty),
										   pt2rshort(bottomstarty));
		topstarty+=topincr;
		bottomstarty+=bottomincr;
		startx++;
	}
	return ERR_OK;
}

Error _HairLine(Bitmap bitmap, pt fromx, pt fromy, pt tox, pt toy)
{
	BitDrawRouts *rout=bitmap->bitdraw;
	short swapy;
	pt swapx, dx;

	fromy=pt2rshort(fromy);
	toy=pt2rshort(toy);
	if (fromy==toy)
	{
		fromx=pt2rshort(fromx);
		tox=pt2rshort(tox);
		if (fromx>tox)
			rout->LineHorizontal(bitmap, fromy, tox, fromx);
		else
			rout->LineHorizontal(bitmap, fromy, fromx, tox);
	} else {
		if (fromy>toy)
		{
			swapx=tox; tox=fromx; fromx=swapx;
			swapy=toy; toy=fromy; fromy=swapy;
		}
		dx=(tox-fromx)/(toy-fromy);
		if (dx>pt_one)
		{
			while (fromy<toy)
			{
				rout->LineHorizontal(bitmap,
							fromy, pt2rshort(fromx), pt2rshort(fromx+dx));
				fromx+=dx;
				fromy++;
			}
		} else if (dx<(-pt_one))
		{
			while (fromy<toy)
			{
				rout->LineHorizontal(bitmap,
							fromy, pt2rshort(fromx+dx), pt2rshort(fromx));
				fromx+=dx;
				fromy++;
			}
		} else {
			while (fromy<toy)
			{
				rout->Pixel(bitmap, pt2rshort(fromx), fromy);
				fromx+=dx;
				fromy++;
			}
		}
	}
	return ERR_OK;
}
