/*
	PROforma, PROgs FOnt & Raster MAnager

	PROforma main routine
	this file should - load the configuration file
					 - initialise the PROforma thing

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 9 September 1993
	23.03.1994 adjusted to load external drivers
	11.11.1994 external drivers can now use DLL Manager for relocation etc.
	21.09.1995 adjusted for runtime configuration
	14.06.1997 PFAllocate & PFRelease routines for less memory fragmentation !
			   + used to allocate charwidth tables, CachedFont entries etc.
*/

#include <io.h>
#include <dev.h>
#include <str.h>
#include <win.h>
#include <job.h>
#include <atomic.h>

#include "gstate.h"
#include "system.h"

#define catch(x) do { if (err=(x)) return err; } while (0)

extern int ResFonts;				/* for sanity checking */

Error PFThingInit();
Error PFConfig(char *line);

static WindowDescriptor report_window={ 350,50,85,80,6,10,0,1,0,7 };

/* some global variable which are initialised/changed here */
Job PROforma;				/* Job id for myself */
Heap myheap;				/* need some heap memory for PROforma */
short InitIsDone=FALSE; 	/* TRUE when initialisation finished */

static Error _allocate(Size size, void **base)
{ return MEMHeapAllocate(myheap,size,base); }
Error PFAllocate(Size size, void **base)
{ return ATOMICCall(_allocate,2,size,base); }
static Error _release(void *base)
{ return MEMHeapRelease(myheap,base); }
Error PFRelease(void *base)
{ return ATOMICCall(_release,1,base); }

static void report(char *format, char *param)
{
	char string[200];
	Window win;

	SMSBeep(2000,20);
	WINOpenFull(&report_window,&win);
	STRFormatPutSafe(string,200,format,param);
	WINPutString(win,string);
	WINPutString(win,"press a key");
	WINGet(win,string);
	JOBZap(MYSELF,ERR_ITNF);
}

Error init(char *param)
{
	Error err;
	char line[100]; 			/* place to read line from PROforma_cfg */
	Channel chid;				/* channel id for fontmap */
	char **params, *path=NULL;
	int i;

	/* fill in jobid of PROforma job */
	JOBMe(&PROforma);

	err=PFThingInit();			/* link in PROforma DLL linkage */
	if (err) return err;

	/* need to call PFConfig to allocate some user memory */
	err=MEMHeapInit(20*1024,&myheap);
	if (!err) err=PFConfig("");
	if (err) report("INSUFFICIENT MEMORY",NULL);
	

	/* find default path for PROforma_cfg file */
	catch( STRBreak(param,&params) );
	for (i=0 ; params[i] ; i++)
	{
		if (STRSameCI("-path",params[i]))
		{
			if (params[i+1])
				path=params[++i];
			else
				return ERR_IPAR;
		}
		else
			path=params[i];
	}

	/* try to open fontmap file */
	err=IOOpenPath("PROforma_cfg",OPEN_READ,path,&chid);
	if (err)
		err=IOOpenPath("PROforma_cfg",OPEN_READ,PATH_PROGDATA,&chid);
	if (err)
		report("can't open \"PROforma_cfg\", the config file",NULL);
	MEMRelease(params);

	while (!IOTest(chid))
	{
		IOGetString(chid, line, 100);
		err=PFConfig(line);
		switch (err)
		{
		case ERR_OK:
			break;
		case ERR_ITNF:
			report("item or file NOT FOUND while processing\n%s",line);
		case ERR_ISYN:
			report("FILE IS NOT AN EXTERNAL DRIVER while processing\n%s",line);
		case ERR_IMEM:
			report("INSUFFICIENT MEMORY for processing\n%s",line);
		case ERR_IPAR:
			report("don't know how to process\n%s",line);
		default:
			report("encountered an UNKNOWN ERROR while processing\n%s",line);
		}
	}
	IOClose(chid);

	/* sanity check - I really do need a resident font or else ... */
	if (!ResFonts) report("no resident font loaded\n",NULL);

	InitIsDone=TRUE;

	/* now suspend this job indefinately */
	while (1) JOBSuspend(MYSELF,FOREVER,NULL);

	return ERR_OK;
}
