/*
	PROforma, PROGS Font & Raster Manager

	test program for v1.10
	starting from a fontlog file,
	print all the characters in that file, to see whatthe outline looks like
	and how effective the hinting is

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	23.10.1996 started
*/

#include "io_h"

#include "PROforma_h"

#define FILENAME "win1_fontlog_1"
#define catch(x) do { if (err=(x)) return err; } while (0)
#define MAX_LINE 300

#define GRID
#define REAL
#define USED

Error init(char *par)
{
	Error err;
	Channel log;
	int charstart;
	Gstate gstate;
	const Bbox fullpage={ short2pt(7*72),short2pt(9*72),short2pt(36),short2pt(36) };
	const ColourRGB blue={0,0,short2pt(100)};
	const ColourRGB lightred={short2pt(100),short2pt(85),short2pt(85)};
	const ColourRGB lightblue={short2pt(66),short2pt(66),short2pt(100)};
	const ColourRGB green={0,short2pt(100),0};
	int passes, charcount=0;
	char line[MAX_LINE];
	int d1,d2,d3,d4,d5,d6,i,j,c;
	double f1,f2,f3,f4,f5,f6;

	LinkPROforma();
	catch( PFGstateInit(*par=='\0'?PF_DRIVER_SCREEN:PF_DRIVER_DEFAULT,NULL,&fullpage,&gstate,&passes) );
	PFPageOriginSet(gstate,-short2pt(72),-short2pt(7*72));
	PFFontScale(gstate,short2pt(10));

	catch( IOOpen(FILENAME,OPEN_READ,&log) );

	while (TRUE)	/* loop for all characters */
	{
		IOPositionGet(log,&charstart);
		for (j=0 ; j<passes ; j++)
		{
			double cfactor=1.0; int cx, cy;
			double rfactor=1.0;

			PFPageClear(gstate);

			/* page is drawn in a few passes */
			/* 1 - determine coordinate system and draw grid */
			/* 2 - draw real outline for character */
			/* 3 - draw adjusted outline */

			/* 1 - determine coordinate system and draw grid */
#ifdef GRID
#undef double2pt
#define double2pt(x) (int)((x)*cfactor)
			PFPathFilled(gstate);
			IOPositionSet(log,charstart);
			catch( IOGetString(log,line,MAX_LINE) );
			while( !MEMSame(12,"new character",line) )
			{
				switch(line[0])
				{
				case 'S':
					STRFormatGet(line,NULL,"SSBW %d %d",&d1,&d2);
					cfactor=d2;
					break;
				case '6':
					STRFormatGet(line,NULL,"6 %f",&f1);
					rfactor=cfactor/f1*7*72*65.536;
					cfactor=7*72*65.536;
					break;
				case 'V':
					PFColourRGB(gstate,&lightred);
					STRFormatGet(line,NULL,"VSTEM %d %d",&d1,&d2);
					PFMoveTo(gstate,double2pt(d1),-short2pt(7*72));
					PFLineR(gstate,0,short2pt(720));
					PFLineR(gstate,double2pt(d2),0);
					PFLineR(gstate,0,-short2pt(720));
					PFPathDraw(gstate);
					break;
				case 'H':
					PFColourRGB(gstate,&lightred);
					STRFormatGet(line,NULL,"HSTEM %d %d",&d1,&d2);
					PFMoveTo(gstate,0,-double2pt(d1));
					PFLineR(gstate,short2pt(720),0);
					PFLineR(gstate,0,-double2pt(d2));
					PFLineR(gstate,-short2pt(720),0);
					PFPathDraw(gstate);
					break;
				case 'v':
					PFColourRGB(gstate,&lightblue);
					STRFormatGet(line,NULL,"vSTEM3 %d %d %d %d %d %d",&d1,&d2,&d3,&d4,&d5,&d6);
					PFMoveTo(gstate,double2pt(d1),-short2pt(7*72));
					PFLineR(gstate,0,short2pt(720));
					PFLineR(gstate,double2pt(d2),0);
					PFLineR(gstate,0,-short2pt(720));
					PFPathDraw(gstate);
					PFMoveTo(gstate,double2pt(d3),-short2pt(7*72));
					PFLineR(gstate,0,short2pt(720));
					PFLineR(gstate,double2pt(d4),0);
					PFLineR(gstate,0,-short2pt(720));
					PFPathDraw(gstate);
					PFMoveTo(gstate,double2pt(d5),-short2pt(7*72));
					PFLineR(gstate,0,short2pt(720));
					PFLineR(gstate,double2pt(d6),0);
					PFLineR(gstate,0,-short2pt(720));
					PFPathDraw(gstate);
					break;
				case 'h':
					PFColourRGB(gstate,&lightblue);
					STRFormatGet(line,&c,"vSTEM3 %d %d %d %d %d %d",&d1,&d2,&d3,&d4,&d5,&d6);
					if (c==6)
					{
						PFMoveTo(gstate,0,-double2pt(d1));
						PFLineR(gstate,short2pt(720),0);
						PFLineR(gstate,0,-double2pt(d2));
						PFLineR(gstate,-short2pt(720),0);
						PFPathDraw(gstate);
						PFMoveTo(gstate,0,-double2pt(d3));
						PFLineR(gstate,short2pt(720),0);
						PFLineR(gstate,0,-double2pt(d4));
						PFLineR(gstate,-short2pt(720),0);
						PFPathDraw(gstate);
						PFMoveTo(gstate,0,-double2pt(d5));
						PFLineR(gstate,short2pt(720),0);
						PFLineR(gstate,0,-double2pt(d6));
						PFLineR(gstate,-short2pt(720),0);
						PFPathDraw(gstate);
					}
					break;
				}
				catch( IOGetString(log,line,MAX_LINE) );
			}
#undef double2pt
#define double2pt(x) (int)((x)*rfactor)
			PFColourRGB(gstate,&green);
			PFPathStroked(gstate);
			PFLineWidth(gstate,short2pt(2));
			for (i=-30 ; i<10 ; i++)
			{
				PFMoveTo(gstate,0,double2pt(i));
				PFLineR(gstate,short2pt(720),0);
				PFPathDraw(gstate);
			}
			for (i=0 ; i<30 ; i++)
			{
				PFMoveTo(gstate,double2pt(i),-short2pt(7*72));
				PFLineR(gstate,0,short2pt(720));
				PFPathDraw(gstate);
			}
			PFLineWidth(gstate,short2pt(1));
			for (i=-30 ; i<10 ; i++)
			{
				PFMoveTo(gstate,0,double2pt(i)+(int)(rfactor/2));
				PFLineR(gstate,short2pt(720),0);
				PFPathDraw(gstate);
			}
			for (i=0 ; i<30 ; i++)
			{
				PFMoveTo(gstate,double2pt(i)+(int)(rfactor/2),-short2pt(7*72));
				PFLineR(gstate,0,short2pt(720));
				PFPathDraw(gstate);
			}
#endif

			/* 2 - draw real outline for character */
#undef double2pt
#define double2pt(x) (int)((x)*cfactor)
#ifdef REAL
			PFColourRGB(gstate,&blue);
			PFLineWidth(gstate,pt_one);
			IOPositionSet(log,charstart);
			catch( IOGetString(log,line,MAX_LINE) );
			while( !MEMSame(12,"new character",line) )
			{
				switch(line[0])
				{
				case 'S':
					STRFormatGet(line,NULL,"SSBW %d",&cx);
					cy=0;
					break;
				case 'M':
					STRFormatGet(line,NULL,"MOVE %d %d",&d1,&d2);
					cx+=d1; cy-=d2;
					break;
				case 'L':
					STRFormatGet(line,NULL,"LINE %d %d",&d1,&d2);
					PFMoveTo(gstate,double2pt(cx),double2pt(cy));
					cx+=d1; cy-=d2;
					PFLineTo(gstate,double2pt(cx),double2pt(cy));
					break;
				case 'C':
					STRFormatGet(line,NULL,"CURVE %d %d %d %d %d %d",&d1,&d2,&d3,&d4,&d5,&d6);
					PFMoveTo(gstate,double2pt(cx),double2pt(cy));
					cx+=d1; cy-=d2;
					PFCurveTo(gstate,double2pt(cx	   ),double2pt(cy	   ),
									 double2pt(cx+d3   ),double2pt(cy-d4   ),
									 double2pt(cx+d3+d5),double2pt(cy-d4-d6));
					cx+=d3+d5; cy-=d4+d6;
					break;
				}
				catch( IOGetString(log,line,MAX_LINE) );
			}
			PFPathDraw(gstate);
#endif

			/* 3 - draw adjusted outline */
#undef double2pt
#define double2pt(x) (int)((x)*rfactor)
#ifdef USED
			PFColourGray(gstate,short2pt(100));
			PFLineWidth(gstate,pt_one);
			IOPositionSet(log,charstart);
			catch( IOGetString(log,line,MAX_LINE) );
			while( !MEMSame(12,"new character",line) )
			{
				switch(line[0])
				{
				case '1':
					STRFormatGet(line,NULL,"1 %f %f",&f1,&f2);
					PFMoveTo(gstate,double2pt(f1),double2pt(f2));
					break;
				case '2':
					STRFormatGet(line,NULL,"2 %f %f",&f1,&f2);
					PFLineTo(gstate,double2pt(f1),double2pt(f2));
					break;
				case '3':
					STRFormatGet(line,NULL,"3 %f %f %f %f %f %f",&f1,&f2,&f3,&f4,&f5,&f6);
					PFCurveTo(gstate,double2pt(f1),double2pt(f2),
									 double2pt(f3),double2pt(f4),
									 double2pt(f5),double2pt(f6));
					break;
				}
				catch( IOGetString(log,line,MAX_LINE) );
			}
			PFPathDraw(gstate);
#endif
			PFColourGray(gstate,short2pt(100));
			PFMoveTo(gstate,0,short2pt(2*72));
			STRFormatPut(line,"char %d",charcount);
			PFStringShow(gstate,line);
			PFPageShow(gstate);
		}
		charcount++;
	}
	return ERR_OK;
}
